﻿//Box2D_07_04.as
package {
	//▼インポート
	import flash.display.Sprite;
	import Box2D.Collision.b2AABB;
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2World;
	import Box2D.Dynamics.b2BodyDef;
	import Box2D.Collision.Shapes.b2CircleDef
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2DebugDraw;
	import flash.events.Event;
	import flash.display.SimpleButton;
	import flash.events.MouseEvent;

	public class Box2D_07_04 extends Sprite {
		//▼定数定義
		//1mを表すピクセル数
		private static const DRAW_SCALE:Number = 100;
		//▼プロパティ定義
		private var _world:b2World;
		private var _bBall:b2Body;
		//▼コンストラクタ定義
		public function Box2D_07_04() {
			//▼物理エンジンのセットアップ
			//物理エンジン適用範囲の設定（ステージの100ピクセル外側）
			var worldRegion:b2AABB = new b2AABB();//矩形作成
			worldRegion.lowerBound.Set(-1, -100);
			worldRegion.upperBound.Set(6.5, 5);
			//重力の設定
			var gravity:b2Vec2 = new b2Vec2(0, 0);
			//範囲・重力・スリープの有無を指定して物理ワールドを作成
			_world = new b2World(worldRegion, gravity, true);
			
			//▼ボール（動体）の生成
			var bdBall:b2BodyDef = new b2BodyDef();
			bdBall.position.Set(5.5 / 2, 2);
			//ShapeDefの設定
			var pdBall:b2CircleDef = new b2CircleDef();
			pdBall.radius = 0.25;
			pdBall.density = 1;//密度（1平方メートルあたりの重量（kg））
			//ボディの生成
			_bBall = _world.CreateBody(bdBall);
			_bBall.CreateShape(pdBall);
			//質量の設定
			_bBall.SetMassFromShapes();
			
			//▼描画設定
			var dd:b2DebugDraw = new b2DebugDraw();
			dd.m_sprite = this;
			dd.m_drawScale = DRAW_SCALE; //1mを100ピクセルにする
			dd.m_fillAlpha = 0.3; //塗りのアルファ値
			dd.SetFlags(b2DebugDraw.e_shapeBit);//物体を表示する設定に
			_world.SetDebugDraw(dd);//物理ワールドにセット
			
			//▼イベントリスナーの追加
			addEventListener(Event.ENTER_FRAME, _enterFrameHandler);
			LinVel_btn.addEventListener(MouseEvent.CLICK, _LinVel_btn_clickHandler);
			AppImp_btn.addEventListener(MouseEvent.CLICK, _AppImp_btn_clickHandler);
			Stop_btn.addEventListener(MouseEvent.CLICK, _Stop_btn_clickHandler);
			Sleep_btn.addEventListener(MouseEvent.CLICK, _Sleep_btn_clickHandler);
			AppFor_btn.addEventListener(MouseEvent.MOUSE_DOWN, _AppFor_btn_mouseDownHandler);
		}
		
		//▼イベントリスナー定義
		private function _enterFrameHandler(eventObj:Event):void {
			_world.Step(1 / 24, 10);//物理シミュレーションの更新
			var ballY:Number = _bBall.GetPosition().y;
			if (_bBall.GetPosition().x > 5.5){
				_bBall.SetXForm(new b2Vec2(0, ballY), _bBall.GetAngle());
			}
		}
		//以下ボタン用イベントリスナー
		private function _LinVel_btn_clickHandler(eventObj:Event):void {
			if (_bBall.IsSleeping()) _bBall.WakeUp();
			var vel:b2Vec2 = _bBall.GetLinearVelocity().Copy();
			vel.Add(new b2Vec2(5, 0));
			_bBall.SetLinearVelocity(vel);
		}
		private function _AppImp_btn_clickHandler(eventObj:Event):void {
			var c:b2Vec2 = _bBall.GetWorldCenter().Copy();
			c.Add(new b2Vec2(0, 0.1));
			_bBall.ApplyImpulse(new b2Vec2(_bBall.GetMass() * 5, 0), c);
		}
		private function _Stop_btn_clickHandler(eventObj:Event):void {
			_bBall.SetLinearVelocity(new b2Vec2(0, 0));
		}
		private function _Sleep_btn_clickHandler(eventObj:Event):void {
			if (!_bBall.IsSleeping()) _bBall.PutToSleep();
		}
		private function _AppFor_btn_mouseDownHandler(eventObj:Event):void {
			addEventListener(Event.ENTER_FRAME, _AppFor_enterFrameHandler);
			stage.addEventListener(MouseEvent.MOUSE_UP, _stage_mouseUpHandler);
		}
		private function _AppFor_enterFrameHandler(eventObj:Event):void {
			var c:b2Vec2 = _bBall.GetWorldCenter().Copy();
			c.Add(new b2Vec2(0, 0.05));
			_bBall.ApplyForce(new b2Vec2(1, 0), c);
		}
		private function _stage_mouseUpHandler(eventObj:Event):void {
			removeEventListener(Event.ENTER_FRAME, _AppFor_enterFrameHandler);
			stage.removeEventListener(MouseEvent.MOUSE_UP, _stage_mouseUpHandler);
		}
	}
}